@@ -30,12 +30,13 @@
 #include "cstitch.h"
 #include "list.h"
 
+extern int debug_level;
+
 /*------- Forward !#*&^@!*#@ Reference ------------------------- */
 int std_solidp();
 int std_contents_equivalentp();
 void std_insert_contents();
 
-
 /*---------------------------------------------------------------
  * Templates for corner-stiched data-structures (see cstitch.h)
  *---------------------------------------------------------------
@@ -102,6 +103,13 @@ void reset_boundaries(root, x1, y1, x2, y2)
     tilist *top = NULL, *bottom = NULL;
     tile *ulhc, *lrhc;
 
+    /* Added by Tim for debugging */
+    if (verify_tile_space(root) == FALSE)
+    {
+	fprintf(stderr, "reset_boundaries: Corrupted tile space! (on entry)\n");
+    }
+    /* end of addition --Tim */
+
     ulhc = find_ULHC(root);
     lrhc = find_LRHC(root);
 
@@ -171,9 +179,10 @@ void reset_boundaries(root, x1, y1, x2, y2)
 
 }
 /*-----------------------------------------------------------------------*/
-void determine_bounding_box(root, x1, y1, x2, y2)
+void determine_bounding_box(root, x1, y1, x2, y2, compress)
     tile *root;
     int *x1, *y1, *x2, *y2;
+    int compress;
 {
     /* Determine the min. bounding box that will contain all of the filled 
        tiles within the tile-space defined by <root>. 
@@ -183,7 +192,9 @@ void determine_bounding_box(root, x1, y1, x2, y2)
     tile *ul, *lr;
 
     /* Reset the bb based on the minimum permissible boundry: */
-    reset_boundaries(root, 0, 0, 0, 0);
+    if (compress)
+       reset_boundaries(root, 0, 0, 0, 0);
+
     ul = find_ULHC(root);
     lr = find_LRHC(root);
 
@@ -662,7 +673,8 @@ tile *insert_tile(start, xorg, yorg, xsize, ysize, what)
     if (verify_tile_space(new) == FALSE)
     {
 	fprintf(stderr, "insert_tile: Corrupted tile space! abort! (on exit)\n");
-	exit();
+	/* exit(1); */
+	return NULL;
     }
     
     return(new);
@@ -1790,6 +1802,10 @@ tilist *helpAreaEnumerate(t, soFar, filterFn, xorg, yorg, maxX, maxY)
     int x = t->x_pos + t->x_len;
     int y = t->y_pos + t->y_len;
 
+    if (debug_level == 5)
+       fprintf(stderr, "t=0x%x (%d, %d)->(%d, %d) in (%d, %d)->(%d, %d) ? ",
+		t, t->x_pos, t->y_pos, x, y, xorg, yorg, maxX, maxY);
+
     /* If <t> is in the specified range, add it to the listing: */
     if ((filterFn(t) != FALSE) && 
 	((((x <= maxX) && (x > xorg)) || 
@@ -1801,7 +1817,12 @@ tilist *helpAreaEnumerate(t, soFar, filterFn, xorg, yorg, maxX, maxY)
 	  ((y >= maxY) && (t->y_pos <= yorg)))))
     {
 	push(t, soFar);		/* Enumerate the little bugger */
+        if (debug_level == 5)
+	   fprintf(stderr, "yes!\n");
     }
+    else
+       if (debug_level == 5)
+	  fprintf(stderr, "no.\n");
     
 
     if (t->tr != NULL)
