@@ -186,6 +186,8 @@ void handle_client(ClientData clientData, XEvent *eventPtr)
 void ghostinit()
 {
 #ifndef HAVE_CAIRO
+   if (areawin->area == NULL) return;
+
    gv = XInternAtom(dpy, "GHOSTVIEW", False);
    gvc = XInternAtom(dpy, "GHOSTVIEW_COLORS", False);
    gvpage = XInternAtom(dpy, "PAGE", False);
@@ -288,7 +290,7 @@ void start_gs()
 #ifndef XC_WIN32
    ret = pipe(std_err);
 #else
-   ret = _pipe(srd_err, pipe_size, pipe_mode);
+   ret = _pipe(std_err, pipe_size, pipe_mode);
 #endif  /* XC_WIN32 */
 #endif
 
