@@ -70,18 +70,12 @@ FILE *svgf;
 
 void svg_printcolor(int passcolor, char *prefix)
 {
-   int i;
-   if (passcolor != DEFAULTCOLOR) {
-      for (i = 0; i < number_colors; i++) {
-         if (colorlist[i].color.pixel == passcolor) break;
-      }
-      if (i < number_colors) {
-         fprintf(svgf, "%s\"#%02x%02x%02x\" ",
+   if ((passcolor < number_colors) && (passcolor != DEFAULTCOLOR)) {
+      fprintf(svgf, "%s\"#%02x%02x%02x\" ",
 		prefix,
-		(colorlist[i].color.red >> 8),
-		(colorlist[i].color.green >> 8),
-		(colorlist[i].color.blue >> 8));
-      }
+		(colorlist[passcolor].color.red >> 8),
+		(colorlist[passcolor].color.green >> 8),
+		(colorlist[passcolor].color.blue >> 8));
    }
 }
 
@@ -278,7 +272,7 @@ void SVGDrawGraphic(graphicptr gp)
     int i;
     char outname[128], *pptr;
     float tscale;
-    int rotation;
+    float rotation;
     int width = xcImageGetWidth(gp->source);
     int height = xcImageGetHeight(gp->source);
 
@@ -304,10 +298,10 @@ void SVGDrawGraphic(graphicptr gp)
 
     tscale = gp->scale * UTopScale();
     rotation = gp->rotation + UTopRotation();
-    if (rotation >= 360) rotation -= 360;
-    else if (rotation < 0) rotation += 360;
+    if (rotation >= 360.0) rotation -= 360.0;
+    else if (rotation < 0.0) rotation += 360.0;
     
-    fprintf(svgf, "<image transform=\"translate(%d,%d) scale(%g) rotate(%d)\"\n",
+    fprintf(svgf, "<image transform=\"translate(%d,%d) scale(%g) rotate(%f)\"\n",
 			ppt.x, ppt.y, tscale, rotation);
     fprintf(svgf, "  width=\"%dpx\" height=\"%dpx\"", width, height);
     fprintf(svgf, " xlink:href=\"%s\">\n", outname);
@@ -524,7 +518,7 @@ void SVGDrawObject(objinstptr theinstance, short level, int passcolor, pushlistp
    
   	  case(LABEL): 
 	    if (level == 0 || TOLABEL(areagen)->pin == False ||
-			(TOLABEL(areagen)->justify & PINVISIBLE))
+			(TOLABEL(areagen)->anchor & PINVISIBLE))
             SVGDrawString(TOLABEL(areagen), curcolor, theinstance);
 	    break;
       }
@@ -551,15 +545,17 @@ void SVGDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
 {
    stringpart *strptr;
    char *textptr;
-   short  fstyle, ffont, tmpjust, baseline, deltay;
+   short  fstyle, ffont, tmpanchor, baseline, deltay;
    int    pos, defaultcolor, curcolor, scolor;
    short  oldx, oldfont, oldstyle;
    int olinerise = 4;
    float  tmpscale = 1.0, natscale = 1.0;
    XPoint newpoint;
    TextExtents tmpext;
+   TextLinesInfo tlinfo;
    short *tabstops = NULL;
    short tabno, numtabs = 0, group = 0;
+   short linenum = 0;
    int open_text, open_span, open_decor;
    XPoint *decorations = NULL;
    short nvals = 0;
@@ -623,8 +619,8 @@ void SVGDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
    UPushCTM();
    UPreMultCTM(DCTM, drawlabel->position, drawlabel->scale, drawlabel->rotation);
 
-   /* check for flip invariance; recompute CTM and justification if necessary */
-   tmpjust = flipadjust(drawlabel->justify);
+   /* check for flip invariance; recompute CTM and anchoring if necessary */
+   tmpanchor = flipadjust(drawlabel->anchor);
 
    /* Note that the Y-scale is inverted or text comes out upside-down.  But we	*/
    /* need to adjust to the Y baseline.						*/
@@ -636,19 +632,22 @@ void SVGDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
    fprintf(svgf, ">\n");
 
    /* "natural" (unscaled) length */
-   tmpext = ULength(drawlabel, localinst, 0, NULL);
+   tlinfo.dostop = 0;
+   tlinfo.tbreak = NULL;
+   tlinfo.padding = NULL;
+   tmpext = ULength(drawlabel, localinst, &tlinfo);
 
-   newpoint.x = (tmpjust & NOTLEFT ?
-       (tmpjust & RIGHT ? -tmpext.maxwidth : -tmpext.maxwidth >> 1) : 0);
-   newpoint.y = (tmpjust & NOTBOTTOM ?
-       (tmpjust & TOP ? -tmpext.ascent : -(tmpext.ascent + tmpext.base) >> 1)
+   newpoint.x = (tmpanchor & NOTLEFT ?
+       (tmpanchor & RIGHT ? -tmpext.maxwidth : -tmpext.maxwidth >> 1) : 0);
+   newpoint.y = (tmpanchor & NOTBOTTOM ?
+       (tmpanchor & TOP ? -tmpext.ascent : -(tmpext.ascent + tmpext.base) >> 1)
 		: -tmpext.base);
 
    /* Pinlabels have an additional offset spacing to pad */
    /* them from the circuit point to which they attach.  */
 
    if (drawlabel->pin) {
-      pinadjust(tmpjust, &(newpoint.x), &(newpoint.y), 1);
+      pinadjust(tmpanchor, &(newpoint.x), &(newpoint.y), 1);
    }
 
    oldx = newpoint.x;
@@ -658,6 +657,16 @@ void SVGDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
    open_span = 0;
    open_decor = 0;
    pos = 0;
+
+   /* Adjust for center or right justification */
+   if (tlinfo.padding != NULL) {
+      if (tmpanchor & JUSTIFYRIGHT)
+	 newpoint.x += tlinfo.padding[linenum];
+      else if (tmpanchor & TEXTCENTERED)
+	 newpoint.x += 0.5 * tlinfo.padding[linenum];
+      linenum++;
+   }
+
    for (strptr = drawlabel->string; strptr != NULL;
 		strptr = nextstringpart(strptr, localinst)) {
 
@@ -838,6 +847,15 @@ void SVGDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
 	    baseline -= BASELINE;
 	    newpoint.y = baseline;
 	    newpoint.x = oldx;
+
+	    if (tlinfo.padding != NULL) {
+	       if (tmpanchor & JUSTIFYRIGHT)
+		  newpoint.x += tlinfo.padding[linenum];
+	       else if (tmpanchor & TEXTCENTERED)
+		  newpoint.x += 0.5 * tlinfo.padding[linenum];
+	       linenum++;
+	    }
+
 	    fprintf(svgf, "<tspan x=\"%d\" y=\"%d\">", newpoint.x, -newpoint.y);
 	    open_span++;
 	    break;
@@ -973,6 +991,7 @@ void SVGDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
    UPopCTM();
 
    if (tabstops != NULL) free(tabstops);
+   if (tlinfo.padding != NULL) free(tlinfo.padding);
 
    /* If there were decorations (underlines, overlines), generate them */
 
@@ -1062,7 +1081,7 @@ OutputSVG(char *filename, Boolean fullscale)
 		pinst->bbox.height + PMARGIN);
 
    fprintf(svgf, "<desc>\n");
-   fprintf(svgf, "XCircuit Version %2.1f\n", PROG_VERSION);
+   fprintf(svgf, "XCircuit Version %s\n", PROG_VERSION);
    fprintf(svgf, "File \"%s\" Page %d\n", xobjs.pagelist[areawin->page]->filename,
 		areawin->page + 1); 
    fprintf(svgf, "</desc>\n");
